# Generated by Django 4.0.2 on 2022-02-09 05:38

from django.db import migrations, models
import django.db.models.deletion
import paper_uploads.models.fields.base


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('paper_uploads', '0008_rename_item_type_collectionitembase_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collectionitembase',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_%(app_label)s.%(class)s_set+', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='fileitem',
            name='file',
            field=paper_uploads.models.fields.base.DynamicStorageFileField(verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='svgitem',
            name='file',
            field=paper_uploads.models.fields.base.DynamicStorageFileField(verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='uploadedfile',
            name='file',
            field=paper_uploads.models.fields.base.DynamicStorageFileField(verbose_name='file'),
        ),
    ]
