# Generated by Django 4.0.1 on 2022-02-17 14:09

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0003_alter_fileitem_resource_name_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fileitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='fileitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='imageitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='imageitem',
            name='modified_at',
        ),
        migrations.RemoveField(
            model_name='svgitem',
            name='created_at',
        ),
        migrations.RemoveField(
            model_name='svgitem',
            name='modified_at',
        ),
        migrations.AddField(
            model_name='collectionitembase',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='created at'),
        ),
        migrations.AddField(
            model_name='collectionitembase',
            name='modified_at',
            field=models.DateTimeField(auto_now=True, verbose_name='changed at'),
        ),
    ]
