# Generated by Django 4.0.4 on 2022-05-25 05:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('paper_uploads', '0008_svgitem_height_svgitem_width_uploadedsvgfile_height_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='concrete_collection_content_type',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='collectionitembase',
            name='concrete_collection_content_type',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='collection',
            name='collection_content_type',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='collectionitembase',
            name='collection_content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype'),
        ),
    ]
