# Generated by Django 3.2.9 on 2021-12-02 18:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0006_auto_20211116_0748'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collection',
            options={'default_permissions': ()},
        ),
    ]
